--
-- AnimRicagi
-- This is the specialization for vehicles which steer with an articulated axis (excavators, loaders ...)
--
-- @author  Stefan Geiger
-- @date  18/08/09
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

AnimRicagi = {};

function AnimRicagi.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Drivable, specializations);
end;

function AnimRicagi:load(xmlFile)
    AnimRicagi.loadAnimRicagi(self, self.xmlFile);
end;

function AnimRicagi:loadAnimRicagi(xmlFile, isInitialized)
	
    local index = getXMLInt(xmlFile, "vehicle.animRicagi#index");
    if index ~= nil then
        local rotSpeed = getXMLFloat(xmlFile, "vehicle.animRicagi#rotSpeed");
        local rotMax = getXMLFloat(xmlFile, "vehicle.animRicagi#rotMax");
        local rotMin = getXMLFloat(xmlFile, "vehicle.animRicagi#rotMin");
        if rotSpeed ~= nil and rotMax ~= nil and rotMin ~= nil then
            rotSpeed = math.rad(rotSpeed);
            rotMax = math.rad(rotMax);
            rotMin = math.rad(rotMin);
            local entry = {};

            entry.anchorActor = Utils.getNoNil(getXMLInt(xmlFile,  "vehicle.animRicagi#anchorActor"), 0);
            if not isInitialized then
                entry.rotNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.animRicagi#rotNode"));
            end;
            entry.rotMax = rotMax;
            entry.rotMin = rotMin;
            entry.rotSpeed = rotSpeed;

            local maxRotTime = rotMax/rotSpeed;
            local minRotTime = rotMin/rotSpeed;
            if minRotTime > maxRotTime then
                local temp = minRotTime;
                minRotTime = maxRotTime;
                maxRotTime = temp;
            end;
            if maxRotTime > self.maxRotTime then
                self.maxRotTime = maxRotTime;
            end;
            if minRotTime < self.minRotTime then
                self.minRotTime = minRotTime;
            end;

            entry.curRot = 0;
            local i = 0;
            entry.rotatingRicagi = {};
            while true do
                local node = Utils.indexToObject(self.components, getXMLString(xmlFile, string.format("vehicle.animRicagi.rotatingRicag(%d)#index", i)));
                if node == nil then
                    break;
                end;
                local rotatingRicag = {};
                rotatingRicag.node = node;
                local rx, ry, rz = getRotation(node);
                rotatingRicag.defRot = {rx, ry, rz};
                rotatingRicag.posRot = Utils.getRadiansFromString(getXMLString(xmlFile, string.format("vehicle.animRicagi.rotatingRicag(%d)#posRot", i)), 3);
                rotatingRicag.negRot = Utils.getRadiansFromString(getXMLString(xmlFile, string.format("vehicle.animRicagi.rotatingRicag(%d)#negRot", i)), 3);
                rotatingRicag.negRotFactor = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.animRicagi.rotatingRicag(%d)#negRotFactor", i)), 1);
                rotatingRicag.posRotFactor = Utils.getNoNil(getXMLFloat(xmlFile, string.format("vehicle.animRicagi.rotatingRicag(%d)#posRotFactor", i)), 1);
                rotatingRicag.invertSteeringAngle = Utils.getNoNil(getXMLBool(xmlFile, string.format("vehicle.animRicagi.rotatingRicag(%d)#invertSteeringAngle", i)), false);
                table.insert(entry.rotatingRicagi, rotatingRicag);
                i = i + 1;
            end;

            self.animRicagi = entry;
        end;
    end;

    self.interpolatedRotatedTime = 0;
end;

function AnimRicagi:delete()
end;

function AnimRicagi:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AnimRicagi:keyEvent(unicode, sym, modifier, isDown)
end;

function AnimRicagi:update(dt)
    if self:getIsActive() then
        if self.animRicagi ~= nil then

            local curRot = self.animRicagi.curRot;
            if self.interpolatedRotatedTime < self.rotatedTime then
                self.interpolatedRotatedTime = math.min(self.rotatedTime, self.interpolatedRotatedTime + math.abs(self.animRicagi.rotSpeed) * dt/500);
            elseif self.interpolatedRotatedTime > self.rotatedTime then
                self.interpolatedRotatedTime = math.max(self.rotatedTime, self.interpolatedRotatedTime - math.abs(self.animRicagi.rotSpeed) * dt/500);
            end;

            local steeringAngle = self.interpolatedRotatedTime * self.animRicagi.rotSpeed;
            if steeringAngle > self.animRicagi.rotMax then
                steeringAngle = self.animRicagi.rotMax;
            elseif steeringAngle < self.animRicagi.rotMin then
                steeringAngle = self.animRicagi.rotMin;
            end;
            if self.updateAnimRicagiRotation ~= nil then
                steeringAngle = self:updateAnimRicagiRotation(steeringAngle, dt);
            end;
            if math.abs(steeringAngle - curRot) > 0.000001 then

                if self.isClient then
                    local percent = 0;
                    if steeringAngle > 0 then
                        percent = steeringAngle / self.animRicagi.rotMax;
                    elseif steeringAngle < 0 then
                        percent = steeringAngle / self.animRicagi.rotMin;
                    end;
                    for i,rotPart in pairs(self.animRicagi.rotatingRicagi) do
                        local rx,ry,rz;
                        if (steeringAngle > 0 and not rotPart.invertSteeringAngle) or (steeringAngle < 0 and rotPart.invertSteeringAngle) then
                            rx,ry,rz = Utils.vector3ArrayLerp(rotPart.defRot, rotPart.posRot, math.min(1.5,percent*rotPart.posRotFactor));
                        else
                            rx,ry,rz = Utils.vector3ArrayLerp(rotPart.defRot, rotPart.negRot, math.min(1.5,percent*rotPart.negRotFactor));
                        end;
                        setRotation(rotPart.node, rx,ry,rz);
                        if self.setMovingToolDirty ~= nil then
                            self:setMovingToolDirty(rotPart.node);
                        end;
                    end;
                end;
            end;

        end;
    end;
end;

function AnimRicagi:draw()
end;

function AnimRicagi:developmentReloadFromXML(xmlFile)
    AnimRicagi.loadAnimRicagi(self, xmlFile, true);
end;

function AnimRicagi:setVehicleRpmUp(dt, isActive)
end;